<?php
namespace WebDB87\Sample;

/**
 * クラスからDocCommentブロックを取得するサンプルスクリプト
 *
 * ちなみにこれはファイル冒頭のDocCommentです。
 *
 * @author USAMI Kenta <tadsan@zonu.me>
 */

include_once __DIR__ . '/vendor/autoload.php';

echo PHP_EOL;

$class = new \ReflectionClass('\WebDB87\Sample\Model\User');
echo $class->getNamespaceName(), PHP_EOL;
echo $class->getDocComment(), PHP_EOL;
echo pretty_print($class->getDocComment()), PHP_EOL;
echo PHP_EOL;

foreach ($class->getMethods() as $method) {
    $doc = preg_replace('/^ +/m', ' ', $method->getDocComment(), -1);
    echo $class->getNamespaceName(), '::', $method->getName(), '()', PHP_EOL;
    echo $doc, PHP_EOL;
    echo pretty_print($class->getDocComment()), PHP_EOL;
    echo PHP_EOL;
}

/**
 * DocCommentをパースして読みやすく表示する
 *
 * @param  string $doc_comment
 * @return string JSON形式にエンコードされた文字列
 */
function pretty_print($doc_comment)
{
    $parser = new \DocBlockReader\Reader($doc_comment);

    return json_encode($parser->getParameters(), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
}
