<?php
namespace WebDB87\Sample;
use WebDB87\Sample\URL;

require_once __DIR__ . '/vendor/autoload.php';

/**
 * PhpStormでの入力補助
 *
 * ComposerとPhpStormが正しくセットアップされていれば補完が機能するようになります
 *
 * @package WebDB87\Sample
 * @author USAMI Kenta
 * @copyright 2015 USAMI Kenta
 * @license https://creativecommons.org/choose/zero/ CC0-1.0
 */

date_default_timezone_set('asia/tokyo');

/**
 * @return \Datetime[]|\ArrayObject
 */
function getHolidays() {
    return new \ArrayObject(array_map(function($n){
        return new \DateTime("$n day ago");
    }, range(7, 1, -1)));
}

$holidays = getHolidays();
// $holidaysは\ArrayObjectのインスタンスです

foreach ($holidays as $day) {
    $day->getTimezone();
    //$day->
    // ↑の行のコメントアウトを削除して > の続きを入力してみてください。
}

$d = $holidays[0];
$d->format('Y-m-d');
//$d->
// ↑の行のコメントアウトを削除して > の続きを入力してみてください。

$holidays->count();
//$holidays->
// ↑の行のコメントアウトを削除して > の続きを入力してみてください。
// ArrayObjectクラスのメソッドが補完可能です
// http://php.net/manual/ja/class.arrayobject.php


// この下はPhpStormとComposerの設定が済んでいない環境では補完が機能しません


/** 引数の一致 */

$area1 = trapezoid(1, 2, 3);
//$area2 = trapezoid();
// コメントアウトを外すと、引数の数が定義と一致しないので警告表示されます


/** メソッドチェイン(fluent interface) */

$obj = new \WebDB87\Sample\AwesomeClass();
$obj->foo()->bar()->buz();
//$obj->
// ↑の行のコメントアウトを削除して > の続きを入力してみてください。


/** @return void */

$v = voidFunc();
// 値を返さない(void)関数の戻り値を変数に代入しています。
// エラーにはならないが、戻り値を受け取る意味がない。


/** @deprecated タグ */

oldFunc();
// @deprecatedな関数を利用しようとしています。


/** @property-read タグ */

$url = URL::fromString('http://gihyo.jp/magazine/wdpress');
var_dump($url->host);
//var_dump($url->);
// ↑の行のコメントアウトを削除して > の続きを入力してみてください。
// $url->hostはマジックメソッドを使って公開しているプロパティです。

$luka = new \WebDB87\Sample\Model\User(3, 'Megurine Luka', 'luka');
var_dump($luka->id);
var_dump($luka->name);
$luka->name = "hoge";
// 残念ながら書き込み不可能なプロパティに書き込もうとしてもPhpStormは警告を発してくれません
// が、実行時にエラーにはなります。
