<?php
namespace WebDB87\Sample;

/**
 * URLを表現するクラス
 *
 * @copyright (c) the PHP Documentation Group
 * @license https://creativecommons.org/licenses/by/3.0/legalcode CC-BY-3.0
 * @author USAMI Kenta <tadsan@zonu.me>
 * @property-read $scheme   string
 * @property-read $host     string
 * @property-read $port     string
 * @property-read $user     string
 * @property-read $pass     string
 * @property-read $path     string
 * @property-read $query    string
 * @property-read $fragment string
 */
final class URL
{
    /** @var array 変更不能な値 */
    private $url;

    /**
     * @param array parse_url()形式の配列
     */
    public function __construct(array $url_array)
    {
        $this->url = $url_array;
    }

    public function __get($name)
    {
        return $this->url[$name];
    }

    public function __isset($name)
    {
        return isset($this->url[$name]);
    }

    /**
     * @param  string $url_string
     * @return URL URLクラスのインスタンスを生成します
     * @throws \DomainException URLとしてパースできない文字列が渡されたとき
     * @link http://php.net/manual/function.parse-url.php
     */
    public static function fromString($url_string)
    {
        $parsed = parse_url($url_string);

        if ($parsed === false) {
            throw new \DomainException;
        }

        return new URL($parsed);
    }

    /**
     * @return array
     */
    public function toArray()
    {
        return $this->url;
    }

    /**
     * @copyright (c) the PHP Documentation Group
     * @license https://creativecommons.org/licenses/by/3.0/legalcode CC-BY-3.0
     * @author thomas at gielfeldt dot com
     * @link http://php.net/manual/function.parse-url.php#106731 unparse_url()を改造した
     * @return string
     */
    public function buildString()
    {
        $scheme   = isset($this->scheme)   ? $this->scheme . '://' : '';
        $host     = isset($this->host)     ? $this->host : '';
        $port     = isset($this->port)     ? ':' . $this->port : '';
        $user     = isset($this->user)     ? $this->user : '';
        $pass     = isset($this->pass)     ? ':' . $this->pass  : '';
        $pass     = (strlen($user) > 0 || strlen($pass) > 0) ? "$pass@" : '';
        $path     = isset($this->path)     ? $this->path : '';
        $query    = isset($this->query)    ? '?' . $this->query : '';
        $fragment = isset($this->fragment) ? '#' . $this->fragment : '';

        return "$scheme$user$pass$host$port$path$query$fragment";
    }
}
